package io.zipcoder.tc_spring_poll_application.dto.error;

import org.springframework.boot.context.config.ResourceNotFoundException;
import org.springframework.context.MessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@ControllerAdvice
public class RestExeptionHandler {
    private MessageSource messageSource;

    @ExceptionHandler(ResourceNotFoundException.class)
    public ResponseEntity<?> ResourceNotFoundError(ResourceNotFoundException rnfe, HttpServletRequest request)
        throws Exception{
        ErrorDetail erros = new ErrorDetail();

        erros.setDetail(rnfe.getMessage());
        erros.setTimeStamp(new Date().getTime());
        erros.setStatus(HttpStatus.NOT_FOUND.value());
        erros.setTitle("Resource not Found" );
        erros.setDeveloperMessage(rnfe.getClass().getName());
        return new ResponseEntity<>(erros,null,HttpStatus.NOT_FOUND);


    }





    @ExceptionHandler(MethodArgumentNotValidException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public @ResponseBody
    ErrorDetail handleValidationError(MethodArgumentNotValidException mave, HttpServletRequest request){
        ErrorDetail errorDetail = new ErrorDetail();

        errorDetail.setTimeStamp(new Date().getTime());
        errorDetail.setStatus(HttpStatus.BAD_REQUEST.value());

        String requestPath = (String) request.getAttribute("java.servlet.error.request_uri");

        if (requestPath == null){
            requestPath = request.getRequestURI();

        }

        errorDetail.setTitle("Bad entry, validation Error");
        errorDetail.setDetail("Input validation fail");
        errorDetail.setDeveloperMessage(mave.getClass().getName());


        List <FieldError> fieldErrors =
                mave.getBindingResult()
                .getFieldErrors();
        for (FieldError field: fieldErrors){
            List <ValidationError> validationErrorList =
                    errorDetail
                            .getErrors()
                            .get(field
                            .getField());


            if (validationErrorList == null){
                validationErrorList = new ArrayList<ValidationError>();

                errorDetail.getErrors()
                        .put(field.getField(),validationErrorList);
            }
            ValidationError validationError = new ValidationError();

            validationError.setCode(field.getCode());
            validationError.setMessage(messageSource.getMessage(field, null));
            validationErrorList.add(validationError);

        }
return errorDetail;
    }


}
